@@ -1,12 +1,15 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+from decimal import Decimal
 from functools import wraps
 
+from trytond.i18n import gettext
 from trytond.model import Workflow, ModelView, fields
+from trytond.model.exceptions import AccessError
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
 
-__all__ = ['ShipmentOut', 'ShipmentOutReturn', 'Move']
+from trytond.modules.product import round_price
 
 
 def process_sale(moves_field):
@@ -28,14 +31,6 @@ def process_sale(moves_field):
 class ShipmentOut(metaclass=PoolMeta):
     __name__ = 'stock.shipment.out'
 
-    @classmethod
-    def __setup__(cls):
-        super(ShipmentOut, cls).__setup__()
-        cls._error_messages.update({
-                'reset_move': ('You cannot reset to draft a move generated '
-                    'by a sale.'),
-                })
-
     @classmethod
     @ModelView.button
     @Workflow.transition('draft')
@@ -43,9 +38,11 @@ class ShipmentOut(metaclass=PoolMeta):
         SaleLine = Pool().get('sale.line')
         for shipment in shipments:
             for move in shipment.outgoing_moves:
-                if (move.state == 'cancel'
+                if (move.state == 'cancelled'
                         and isinstance(move.origin, SaleLine)):
-                    cls.raise_user_error('reset_move')
+                    raise AccessError(
+                        gettext('sale.msg_sale_move_reset_draft',
+                            move=move.rec_name))
 
         return super(ShipmentOut, cls).draft(shipments)
 
@@ -58,7 +55,7 @@ class ShipmentOut(metaclass=PoolMeta):
 
     @classmethod
     @ModelView.button
-    @Workflow.transition('done')
+    @Workflow.transition('cancelled')
     @process_sale('outgoing_moves')
     def cancel(cls, shipments):
         super(ShipmentOut, cls).cancel(shipments)
@@ -67,14 +64,6 @@ class ShipmentOut(metaclass=PoolMeta):
 class ShipmentOutReturn(metaclass=PoolMeta):
     __name__ = 'stock.shipment.out.return'
 
-    @classmethod
-    def __setup__(cls):
-        super(ShipmentOutReturn, cls).__setup__()
-        cls._error_messages.update({
-                'reset_move': ('You cannot reset to draft a move generated '
-                    'by a sale.'),
-                })
-
     @classmethod
     @ModelView.button
     @Workflow.transition('draft')
@@ -82,9 +71,11 @@ class ShipmentOutReturn(metaclass=PoolMeta):
         SaleLine = Pool().get('sale.line')
         for shipment in shipments:
             for move in shipment.incoming_moves:
-                if (move.state == 'cancel'
+                if (move.state == 'cancelled'
                         and isinstance(move.origin, SaleLine)):
-                    cls.raise_user_error('reset_move')
+                    raise AccessError(
+                        gettext('sale.msg_sale_move_reset_draft',
+                            move=move.rec_name))
 
         return super(ShipmentOutReturn, cls).draft(shipments)
 
@@ -165,18 +156,45 @@ class Move(metaclass=PoolMeta):
             category = self.origin.unit.category.id
         return category
 
+    def get_cost_price(self, product_cost_price=None):
+        pool = Pool()
+        SaleLine = pool.get('sale.line')
+        Sale = pool.get('sale.sale')
+        # For return sale's move use the cost price of the original sale
+        if (isinstance(self.origin, SaleLine)
+                and self.origin.quantity < 0
+                and self.from_location.type != 'storage'
+                and self.to_location.type == 'storage'
+                and isinstance(self.origin.sale.origin, Sale)):
+            sale = self.origin.sale.origin
+            cost = Decimal(0)
+            qty = Decimal(0)
+            for move in sale.moves:
+                if (move.state == 'done'
+                        and move.from_location.type == 'storage'
+                        and move.to_location.type == 'customer'
+                        and move.product == self.product):
+                    move_quantity = Decimal(str(move.internal_quantity))
+                    cost_price = move.get_cost_price(
+                        product_cost_price=move.cost_price)
+                    qty += move_quantity
+                    cost += cost_price * move_quantity
+            if qty:
+                product_cost_price = round_price(cost / qty)
+        return super().get_cost_price(product_cost_price=product_cost_price)
+
     @property
     def origin_name(self):
         pool = Pool()
         SaleLine = pool.get('sale.line')
         name = super(Move, self).origin_name
-        if isinstance(self.origin, SaleLine):
-            name = self.origin.sale.reference or self.origin.sale.rec_name
+        if isinstance(self.origin, SaleLine) and self.origin.id >= 0:
+            name = self.origin.sale.rec_name
         return name
 
     @classmethod
     @ModelView.button
-    @Workflow.transition('cancel')
+    @Workflow.transition('cancelled')
     @process_sale_move
     def cancel(cls, moves):
         super(Move, cls).cancel(moves)
